
//////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2024 Autodesk, Inc.  All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.   
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __STDANSITAPER_H
#define __STDANSITAPER_H

///<summary>This is class AcmTaperStdANSI.</summary>
class STDIMP_DLLSHARE AcmTaperStdANSI : public AcmTaperStd
{
public:
  ACRX_DECLARE_MEMBERS(AcmTaperStdANSI);
  AcmTaperStdANSI();
  virtual ~AcmTaperStdANSI();
  ///<summary>Gets the information about the standard name and revision.</summary>
  virtual Acm::ErrorStatus getRevision(CString& rev, Adesk::UInt16& year) const;
  ///<summary>Provides the basic interface for the AcmStandardManager and hence determines whether AcmSymbolSTD can be used for a particular standard.</summary>
  virtual Adesk::Boolean   isCompatibleTo(const AcmStandard* pStd) const;
  virtual void setDefaultSubSymbol(AcmCTaper* pData) const;
  ///<summary>This function is used to get default offered symbols according to different TaperSlopeType.</summary>
  ///<param name="symbols">Return the required symbol types.</param>
  ///<param name="type">Set the taper slope type that user wants to get. The default value is AcmTaperStd::kAllType. 
  ///Other two options are AcmTaperStd::kTaperType and AcmTaperStd::kSlopeType.</param>
  virtual void getDefaultOfferedSymbols(AcDbIntArray& symbols,const AcmTaperStd::OfferedSymType type) const;
};

#endif // __STDANSITAPER_H