//////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2024 Autodesk, Inc.  All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.   
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __STDCSNNOTE_H
#define __STDCSNNOTE_H
///<summary>This is class AmdtNoteStdCSN.</summary>
class STDIMP_DLLSHARE AmdtNoteStdCSN : public AmdtNoteStd
{
public:
  ACRX_DECLARE_MEMBERS(AmdtNoteStdCSN);
  AmdtNoteStdCSN();
  virtual ~AmdtNoteStdCSN();             
  ///<summary>Gets the information about the standard name and revision.</summary>
  virtual Acm::ErrorStatus getRevision(CString& rev, Adesk::UInt16& year) const;
  ///<summary>Provides the basic interface for the AcmStandardManager and hence determines whether 
  ///AcmSymbolSTD can be used for a particular standard.</summary>
  virtual Adesk::Boolean   isCompatibleTo(const AcmStandard* pStd) const;
  ///<summary>Returns true if the symbol standard is created for a particular draft standard.</summary>
  ///<param name="pStd">The draft standard.</param>
  virtual Adesk::Boolean   isCreatedFor(const AcmStandard* pStd) const;
  virtual AmdtNoteStd::attachPoint defaultAttachPoint() const;
  ///<summary>Sets the factory default values.</summary>
  virtual Adesk::Boolean setFactoryDefaults();
  virtual void getSubEnts(const AmdtCNote* pSymb,AcmCAuxEntArray& subents) const;
};                      

#endif // __STDCSNNOTE_H