//////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2024 Autodesk, Inc.  All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.   
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __STDDINEDGE_H
#define __STDDINEDGE_H

///<summary>This is class AmdtEdgeStdDIN.</summary>
class STDIMP_DLLSHARE AmdtEdgeStdDIN : public AmdtEdgeStdISO
{
public:
  ACRX_DECLARE_MEMBERS(AmdtEdgeStdDIN);
  AmdtEdgeStdDIN();
  virtual ~AmdtEdgeStdDIN();
  ///<summary>Gets the information about the standard name and revision.</summary>
  virtual Acm::ErrorStatus getRevision(CString& rev, Adesk::UInt16& year) const;
  ///<summary>Provides the basic interface for the AcmStandardManager and hence determines whether 
  ///AcmSymbolSTD can be used for a particular standard.</summary>
  virtual Adesk::Boolean   isCompatibleTo(const AcmStandard* pStd) const;
  virtual void initToEdit();
  virtual void initToEditOriginal();
  ///<summary>Sets the factory default values.</summary>
  virtual Adesk::Boolean setFactoryDefaults();
  virtual Acm::ErrorStatus checkVisibility(AmdtCEdge* pData) const;
  virtual Acm::ErrorStatus initFields(AmdtCEdge* pData) const;
  virtual Acm::ErrorStatus validityFields(AmdtCEdge *pData,
    bool setBoundary = false,
    int dlgIndex = -1);
  virtual void getSubEnts(const AmdtCEdge* pSubSym,
    AcmCAuxEntArray& subents,
    bool corrSymbOnRightSide = false) const;
  ///<summary>It returns true if the edge symbol is drawn above the arrow landing.</summary>
  virtual bool isSymAboveLanding() const;

};                      

///<summary>This is class AmdtEdgeStdDIN13715.</summary>
class STDIMP_DLLSHARE AmdtEdgeStdDIN13715 : public AmdtEdgeStdISO13715
{
public:
  ACRX_DECLARE_MEMBERS(AmdtEdgeStdDIN13715);
  AmdtEdgeStdDIN13715();
  virtual ~AmdtEdgeStdDIN13715();
  ///<summary>Gets the information about the standard name and revision.</summary>
  virtual Acm::ErrorStatus getRevision(CString& rev, Adesk::UInt16& year) const;  
  ///<summary>Provides the basic interface for the AcmStandardManager and hence determines whether 
  ///AcmSymbolSTD can be used for a particular standard.</summary>
  virtual Adesk::Boolean   isCompatibleTo(const AcmStandard* pStd) const; 
  ///<summary>Sets the factory default values.</summary>
  virtual Adesk::Boolean setFactoryDefaults(); 
  ///<summary>Get the symbol's standard number string.</summary>
  ///<param name="rev">standard number.</param>
  virtual void getStandardNum(CString& rev) const;
};                      

#endif // __STDDINEDGE_H